#!/usr/bin/python
#backup_db_keeper.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time

#------------------------------------------------------------------------------ 

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup_rebuilder.py')
    
import lib.dhnio as dhnio
import lib.misc as misc
import lib.contacts as contacts
import lib.settings as settings
import lib.transport_control as transport_control
import lib.dhnpacket as dhnpacket
import lib.commands as commands
import lib.dhncrypto as dhncrypto
import lib.automats as automats
from lib.automat import Automat

import dhnblock
import p2p_connector


_BackupDBKeeper = None
   
#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _BackupDBKeeper
    if _BackupDBKeeper is None:
        _BackupDBKeeper = BackupDBKeeper('backup_db_keeper', 'AT_STARTUP', 4)
    if event is not None:
        _BackupDBKeeper.automat(event, arg)
    return _BackupDBKeeper
    
#------------------------------------------------------------------------------ 

class BackupDBKeeper(Automat):
    timers = {'timer-1sec':     (1,     ['RESTART']),
              'timer-30sec':    (30,    ['RESTART', 'REQUEST', 'SENDING']),
              'timer-1hour':    (60*60, ['READY']),}
    
    def init(self):
        self.requestedSuppliers = set()
        self.sentSuppliers = set()
        self.lastRestartTime = 0

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state is 'AT_STARTUP':
            if event == 'restart' :
                self.state = 'RESTART'
            elif event == 'init' :
                self.state = 'READY'
        #---RESTART---
        elif self.state is 'RESTART':
            if event == 'timer-1sec' and self.isTimePassed(arg) and p2p_connector.A().state is 'CONNECTED' :
                self.state = 'REQUEST'
                self.doSuppliersRequestDBInfo(arg)
                self.doRememberTime(arg)
            elif event == 'timer-30sec' :
                self.state = 'READY'
        #---REQUEST---
        elif self.state is 'REQUEST':
            if event == 'restart' :
                self.state = 'RESTART'
            elif ( event == 'incoming-db-info' and self.isAllSuppliersResponded(arg) ) or event == 'timer-30sec' :
                self.state = 'SENDING'
                self.doSuppliersSendDBInfo(arg)
        #---SENDING---
        elif self.state is 'SENDING':
            if event == 'restart' :
                self.state = 'RESTART'
            elif ( event == 'db-info-acked' and self.isAllSuppliersAcked(arg) ) or event == 'timer-30sec' :
                self.state = 'READY'
        #---READY---
        elif self.state is 'READY':
            if event == 'timer-1hour' or event == 'restart' :
                self.state = 'RESTART'

    def isAllSuppliersResponded(self, arg):
        return len(self.requestedSuppliers) == 0
            
    def isAllSuppliersAcked(self, arg):
        return len(self.sentSuppliers) == 0

    def isTimePassed(self, arg):
        return time.time() - self.lastRestartTime > settings.BackupDBSynchronizeDelay()
    
    def doRememberTime(self, arg):
        self.lastRestartTime = time.time()        
    
    def doSuppliersRequestDBInfo(self, arg):
        dhnio.Dprint(4, 'backup_db_keeper.doSuppliersRequestDBInfo')
        # packetID_ = settings.BackupInfoFileName()
        packetID = settings.BackupInfoEncryptedFileName()
        for supplierId in contacts.getSupplierIDs():
            if supplierId:
                transport_control.RemoveInterest(supplierId, packetID)
        self.requestedSuppliers.clear()
        Payload = ''
        localID = misc.getLocalID()
        for supplierId in contacts.getSupplierIDs():
            if not supplierId:
                continue
            newpacket = dhnpacket.dhnpacket(commands.Retrieve(), localID, localID, packetID, Payload, supplierId)
            transport_control.outboxAck(newpacket)
            transport_control.RegisterInterest(self.SupplierResponse, supplierId, packetID)
            self.requestedSuppliers.add(supplierId)

    def doSuppliersSendDBInfo(self, arg):
        dhnio.Dprint(4, 'backup_db_keeper.doSuppliersSendDBInfo')
        packetID = settings.BackupInfoEncryptedFileName()
        for supplierId in contacts.getSupplierIDs():
            if supplierId:
                transport_control.RemoveInterest(supplierId, packetID)
        self.sentSuppliers.clear()
        src = dhnio.ReadBinaryFile(settings.BackupInfoFileFullPath())
        localID = misc.getLocalID()
        block = dhnblock.dhnblock(localID, packetID, 0, dhncrypto.NewSessionKey(), dhncrypto.SessionKeyType(), 'True', src,)
        Payload = block.Serialize() 
        for supplierId in contacts.getSupplierIDs():
            if not supplierId:
                continue
            newpacket = dhnpacket.dhnpacket(commands.Data(), localID, localID, packetID, Payload, supplierId)
            transport_control.outboxAck(newpacket)
            transport_control.RegisterInterest(self.SupplierAcked, supplierId, packetID)
            self.sentSuppliers.add(supplierId)
            # dhnio.Dprint(6, 'backup_db_keeper.doSuppliersSendDBInfo to %s' % supplierId)

    def SupplierResponse(self, packet):
        self.requestedSuppliers.discard(packet.OwnerID)
        dhnio.Dprint(6, 'backup_db_keeper.SupplierResponse %s, %d more suppliers' % (packet.OwnerID, len(self.requestedSuppliers)))

    def SupplierAcked(self, packet):
        self.sentSuppliers.discard(packet.OwnerID)
        dhnio.Dprint(6, 'backup_db_keeper.SupplierAcked %s, %d more suppliers' % (packet.OwnerID, len(self.sentSuppliers)))
        self.automat('db-info-acked', packet.OwnerID)
    
#    def SupplierFailed(self, remoteID, packetID, why):
#        self.sentSuppliers.discard(remoteID)
#        dhnio.Dprint(6, 'backup_db_keeper.SupplierFailed %s, %d more suppliers' % (remoteID, len(self.sentSuppliers)))
#        self.automat('db-info-acked', remoteID)





#!/usr/bin/python
#transport_skype.py

_Skype4PyInstalled = False
try:
    import Skype4Py
    _Skype4PyInstalled = True
except:
    pass


import os
import sys
import time
import base64
import tempfile
import random
import types

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in transport_skype.py')

from twisted.internet import threads
from twisted.internet.defer import Deferred


import transport_control
import dhnio
import misc
import packetid


_SkypeObject = None
_ApplicationName = 'dhn-transport'
_SkypeStatus = None
_LocalSkypeName = ''

_AttachNotifier = None
_ReceiveNotifier = None
_SentNotifier = None

_PacketSize = 1024*8
_OutboxQueue = []
_Buffer = ''
_CurrentPacket = None

_InboxFilesNames = {}
_InboxFilesSize = {}


#-------------------------------------------------------------------------------

def SetReceiveNotifier(notify_func = None):
    global _ReceiveNotifier
    _ReceiveNotifier = notify_func

def SetSentNotifier(notify_func = None):
    global _SentNotifier
    _SentNotifier = notify_func

def SetAttachNotifier(notify_func = None):
    global _AttachNotifier
    _AttachNotifier = notify_func

#-------------------------------------------------------------------------------


def _Try2Attach():
    global _SkypeObject
    if _SkypeObject is None:
        return
    try:
        _SkypeObject.Attach()
    except:
        dhnio.Dprint(6, 'transport_skype._Try2Attach Attachment Failed')
        return

def _OnAttachmentStatus(status):
    global _SkypeObject
    global _ApplicationName
    global _Skype4PyInstalled
    global _SkypeStatus
    global _LocalSkypeName
    global _AttachNotifier

    if not _Skype4PyInstalled:
        return
    _SkypeStatus = status
    dhnio.Dprint(14, 'transport_skype._OnAttachmentStatus ' + str(status))
    if _SkypeObject is None:
        return
    if status == Skype4Py.apiAttachAvailable:
        reactor.callInThread(_Try2Attach)
        dhnio.Dprint(14, 'transport_skype._OnAttachmentStatus Attach')
    elif status == Skype4Py.apiAttachSuccess:
        try:
            _SkypeObject.Application(_ApplicationName).Create()
            _LocalSkypeName = _SkypeObject.CurrentUserHandle.split(':')[0]
        except:
            dhnio.Dprint(6, 'transport_skype._OnAttachmentStatus ERROR Application create Failed')
            dhnio.DprintException()
            return
        dhnio.Dprint(14, 'transport_skype._OnAttachmentStatus dhn-transport application created')
        if _AttachNotifier is not None:
            _AttachNotifier(status, _LocalSkypeName)
    elif status == Skype4Py.apiAttachRefused:
        dhnio.Dprint(6, 'transport_skype._OnAttachmentStatus Attach Refused')
    elif status == Skype4Py.apiAttachNotAvailable:
        dhnio.Dprint(6, 'transport_skype._OnAttachmentStatus Skype not available')


def connect(skype_name, src):
    global _Skype4PyInstalled
    global _Buffer
    global _SkypeObject
    global _ApplicationName
    dhnio.Dprint(14, 'transport_skype.connect to ' + str(skype_name))
    if not _Skype4PyInstalled:
        dhnio.Dprint(4, 'transport_skype.connect Skype4Py not installed')
        return
    if _SkypeObject is None:
        dhnio.Dprint(4, 'transport_skype.connect skype not ready')
        return
    _Buffer = base64.b64encode(src) + ' '
    try:
        _SkypeObject.Application(_ApplicationName).Connect(skype_name, False)
    except:
        dhnio.Dprint(4, 'transport_skype.connect to %s FAILED' % str(skype_name))
        dhnio.DprintException()
        buffer_fail()
        return

def _OnApplicationConnecting(app, users):
    global _Buffer
    global _SkypeObject
    global _ApplicationName
    try:
        streams = _SkypeObject.Application(_ApplicationName).Streams
    except:
        dhnio.Dprint(1, 'transport_skype._OnApplicationConnecting NETERROR')
        dhnio.DprintException()
        buffer_fail()
        return
    if streams.Count == 0:
        return
    dhnio.Dprint(14, 'transport_skype._OnApplicationConnecting ')
    at_least_one_sent = False
    for stream in streams:
        try:
            skype_name = stream.Handle.split(':')[0]
        except:
            dhnio.Dprint(1, 'transport_skype._OnApplicationConnecting NETERROR wrong stream handle')
            continue
        if _stream_write(stream, _Buffer, skype_name) > 0:
            at_least_one_sent = True
    if at_least_one_sent:
        buffer_sent()
    else:
        buffer_fail()

def _stream_write(stream, data, skype_name):
    global _SentNotifier
    total_length = 0
    for part in data.split(' '):
        try:
            stream.Write(str(len(part))+':'+str(part))
        except:
            dhnio.Dprint(4, 'transport_skype._stream_write  FAIL sending to ' + str(skype_name))
            transport_control.log('skype', 'sending to %s failed' % str(skype_name))
            continue
        total_length += len(part)
    transport_control.log('skype', 'sending to %s done' % str(skype_name))
    dhnio.Dprint(12, 'transport_skype._stream_write %s bytes sent to %s' % (str(total_length), str(skype_name)))
    if _SentNotifier is not None:
        _SentNotifier(skype_name, total_length)
    return total_length

def _OnApplicationStreams(app, streams):
    dhnio.Dprint(14, 'transport_skype._OnApplicationStreams ')
    if streams.Count > 1:
        dhnio.Dprint(4, 'transport_skype._OnApplicationStreams WARNING streams.Count='+str(streams.Count))
##        streams[1].Disconnect()

def _OnApplicationReceiving(app, streams):
    dhnio.Dprint(14, 'transport_skype._OnApplicationReceiving')
    transport_control.log('skype', 'incomming connection')
    if streams.Count == 0:
        return
    if streams[0].DataLength == 0:
        return
    for stream in streams:
        _stream_read(stream)

def _stream_read(stream):
    global _ReceiveNotifier
    dhnio.Dprint(14, 'transport_skype._stream_read')
    try:
        skype_name = stream.Handle.split(':')[0]
    except:
        dhnio.Dprint(1, 'transport_skype._stream_read NETERROR wrong stream handle')
        return
    total_length = 0
    try:
        parts = stream.Read()
    except:
        transport_control.log('skype', 'receiving from %s failed' % str(skype_name))
        dhnio.Dprint(4, 'transport_skype._stream_read  FAILED receiving from ' + str(skype_name))
        return
    for part in parts.split(' '):
        if len(part.strip()) == 0:
            continue
        try:
            (lengthStr, data) = part.split(':')
        except:
            dhnio.Dprint(1, 'transport_skype._stream_read NETERROR wrong data ' + str(part))
            continue
        try:
            lengthI = int(lengthStr)
        except:
            dhnio.Dprint(1, 'transport_skype._stream_read NETERROR wrong data ' + str(lengthStr))
            continue
        if lengthI != len(data):
            dhnio.Dprint(1, 'transport_skype._stream_read NETERROR wrong data size')
            continue
        if lengthI == 0:
            continue
        try:
            data_decoded = base64.b64decode(data)
        except:
            dhnio.Dprint(1, 'transport_skype._stream_read NETERROR decoding data')
            continue

        total_length += lengthI

        packet = unserialize(data_decoded)
        if packet is None:
            dhnio.Dprint(1, 'transport_skype._stream_read ERROR unserialize incomming data')
            continue

        inbox(packet)

    if _ReceiveNotifier is not None:
        _ReceiveNotifier(skype_name, total_length)


#-------------------------------------------------------------------------------

def get_user_status(skype_name):
    global _SkypeObject
    if _SkypeObject is None:
        return
    user = _SkypeObject.User(Username = skype_name)
    return user.OnlineStatus

#-------------------------------------------------------------------------------


class SkypeListener(Deferred):
    def stopListening(self):
        global _SkypeObject
        if _SkypeObject is None:
            return
        _SkypeObject.OnApplicationReceiving = None

def _wait4receive(d, count):
    global _SkypeStatus
    dhnio.Dprint(12, 'transport_skype._wait4receive ' + str(count))
    if _SkypeStatus == Skype4Py.apiAttachSuccess:
        d.callback('')
        return
    if count > 60:
        d.errback('')
        return
    reactor.callLater(5, _wait4receive, d, count+1)

def inbox(packet):
    global _InboxFilesNames
    global _InboxFilesSize
    dhnio.Dprint(12, 'transport_skype.inbox ' + str(packet))

    fd, filename = tempfile.mkstemp(".dhn-skype-in", "%s-%s-"% (str(packet.packet_id), str(packet.file_id)))
    os.write(fd, packet.data)
    try:
        fd.flush()
        os.close(fd)
    except:
        pass

    if not _InboxFilesNames.has_key(packet.file_id):
        _InboxFilesNames[packet.file_id] = list()
        _InboxFilesSize[packet.file_id] = 0
    _InboxFilesNames[packet.file_id].append(filename)
    _InboxFilesSize[packet.file_id] += len(packet.data)
    if _InboxFilesSize[packet.file_id] >=  packet.total_size:
        assemble(packet.file_id, packet.skype_name_from)

def assemble(file_id, skype_name_from):
    global _InboxFilesNames
    global _InboxFilesSize
    dhnio.Dprint(14, 'transport_skype.assemble file_id:%s from %s' % (str(file_id), skype_name_from))

    files_list = _InboxFilesNames[file_id]
    files_list.sort()

    fd, filename = tempfile.mkstemp(".dhn-skype-in")
    for part_filename in files_list:
        src = dhnio.ReadBinaryFile(part_filename)
        os.write(fd, src)
        if os.access(part_filename, os.W_OK):
            try:
                os.remove(part_filename)
            except:
                dhnio.Dprint(1, 'transport_skype.assemble ERROR removing ' + part_filename)
    try:
        fd.flush()
        os.close(fd)
    except:
        pass
    _InboxFilesNames.pop(file_id, None)
    _InboxFilesSize.pop(file_id, None)
    transport_control.receiveStatusReport(filename, "finished",
        'skype', skype_name_from)
    transport_control.log('skype', 'finish receiving from ' + str(skype_name_from))

def receive():
    global _Skype4PyInstalled
    global _SkypeObject
    global _SkypeStatus
    dhnio.Dprint(14, 'transport_skype.receive')
    if not _Skype4PyInstalled:
        dhnio.Dprint(4, 'transport_skype.receive Skype4Py not installed')
        return None, None
    if _SkypeObject is None:
        dhnio.Dprint(4, 'transport_skype.receive skype not ready yet')
        return None, None
    _SkypeObject.OnApplicationReceiving = _OnApplicationReceiving
    d = Deferred()
    if _SkypeStatus == Skype4Py.apiAttachSuccess:
        d.callback('')
    else:
        reactor.callLater(5, _wait4receive, d, 0)
    return d, SkypeListener()

#-------------------------------------------------------------------------------

def sendshort(skype_name, filename):
    try:
        fin = open(filename, 'rb')
        src = fin.read()
        fin.close()
    except:
        dhnio.Dprint(1, 'transport_skype.sendshort ERROR reading ' + str(filename))
        return
    connect(skype_name, src)

#-------------------------------------------------------------------------------

class Packet:
    def __init__(self,
            skype_name_to,
            skype_name_from,
            file_id,
            packet_id,
            total_size,
            data):
        self.skype_name_to = skype_name_to
        self.skype_name_from = skype_name_from
        self.file_id = file_id
        self.packet_id = packet_id
        self.total_size = total_size
        self.data = data
        dhnio.Dprint(14, str(self))

    def serialize(self):
        return misc.ObjectToString(self)

    def __str__(self):
        return 'Packet to %s from %s file_id:%s packet_id:%s with %s from %s bytes' % (
        self.skype_name_to,
        self.skype_name_from,
        self.file_id,
        self.packet_id,
        str(len(self.data)),
        str(self.total_size))

def unserialize(data):
    newobject = misc.StringToObject(data)
    if type(newobject) != types.InstanceType:
        dhnio.Dprint(1, "transport_skype.unserialize ERROR not an instance")
        return None
    if newobject.__class__ != Packet:
        dhnio.Dprint(1, "transport_skype.unserialize ERROR not a Packet:" + str(newobject.__class__))
        return None
    return newobject

def send(skype_name, filename):
    global _PacketSize
    global _LocalSkypeName
    src = dhnio.ReadBinaryFile(filename)

    total_size = len(src)
    file_id = packetid.UniqueID()
    packet_id = 0
    i = 0
    while True:
        buf = src[i:i + _PacketSize]
        packet = Packet(
            skype_name,
            _LocalSkypeName,
            file_id,
            packet_id,
            total_size,
            buf)
        outbox(packet)
        packet_id += 1
        i += _PacketSize
        if i >= len(src):
            break

def outbox(packet):
    global _OutboxQueue
    _OutboxQueue.append(packet)

def buffer_sent():
    global _Buffer
    global _CurrentPacket
    dhnio.Dprint(14, 'transport_skype.buffer_sent')
    packet = unserialize(base64.b64decode(_Buffer.strip()))
    if packet is None:
        return
    if _CurrentPacket is None:
        return
    if _CurrentPacket.packet_id == packet.packet_id and _CurrentPacket.file_id == packet.file_id:
        _OutboxQueue.pop(0)
        _CurrentPacket = None

def buffer_fail():
    global _Buffer
    global _CurrentPacket
    dhnio.Dprint(14, 'transport_skype.buffer_fail')
    packet = unserialize(base64.b64decode(_Buffer.strip()))
    if packet is None:
        return
    if _CurrentPacket is None:
        return
    if _CurrentPacket.packet_id == packet.packet_id and _CurrentPacket.file_id == packet.file_id:
        _CurrentPacket = None

def process_sending_queue():
    global _OutboxQueue
    global _CurrentPacket
    global _SkypeObject

    if not _Skype4PyInstalled:
        return

    reactor.callLater(0.01, process_sending_queue)

    if _SkypeObject is None:
        return
    if _SkypeStatus != Skype4Py.apiAttachSuccess:
        return
    if len(_OutboxQueue) == 0:
        return
    if _CurrentPacket is not None:
        return

    _CurrentPacket = _OutboxQueue[0]
    connect(_CurrentPacket.skype_name_to, _CurrentPacket.serialize())


#-------------------------------------------------------------------------------


def shutdown():
    global _Skype4PyInstalled
    global _SkypeObject
    global _ApplicationName
    if not _Skype4PyInstalled:
        dhnio.Dprint(4, 'transport_skype.shutdown Skype4Py not installed')
        return
    dhnio.Dprint(4, 'transport_skype.shutdown')
    if _SkypeObject is not None:
        try:
            _SkypeObject.Application(_ApplicationName).Delete()
        except:
            dhnio.Dprint(6, 'transport_skype.shutdown Application removing failed')
        del _SkypeObject
        _SkypeObject = None

def init():
    global _SkypeObject
    if not _Skype4PyInstalled:
        dhnio.Dprint(4, 'transport_skype.init Skype4Py not installed')
        return
    dhnio.Dprint(4, 'transport_skype.init')

    if _SkypeObject is not None:
        return
    _SkypeObject = Skype4Py.Skype()
    _SkypeObject.Timeout = 5.0
    _SkypeObject.OnAttachmentStatus = _OnAttachmentStatus
    _SkypeObject.OnApplicationStreams = _OnApplicationStreams
    _SkypeObject.OnApplicationConnecting = _OnApplicationConnecting

    reactor.callInThread(_Try2Attach)

    reactor.addSystemEventTrigger('before', 'shutdown', shutdown)

    reactor.callLater(0, process_sending_queue)


#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------


def cmd_send(skype_name, filename):
    def _attach_func(status, local_skype_name):
        send(skype_name, filename)
    SetAttachNotifier(_attach_func)

def cmd_sendshort(skype_name, filename):
    sendshort(skype_name, filename)

def cmd_receive():
    receive()

_Counter = 0
def cmd_loopsend(skype_name, filename, timeout):
    global _Counter
    _Counter += 1
    send(skype_name, filename)
    reactor.callLater(timeout, cmd_loopsend, skype_name, filename, timeout)

#-------------------------------------------------------------------------------

_RandomData = ''
_Bytes_sent = 0
_Start_tm = time.time()
def _sent_func(name, bytes):
    global _Start_tm
    global _Bytes_sent
    _Bytes_sent += bytes
    dt = time.time() - _Start_tm
    if dt > 0:
        speed = float(bytes/1024.0)/float(dt)
        print speed, 'Kb/sec'
##    _Start_tm = time.time()
##    reactor.callLater(0, _datasend, name, _RandomData)
def _loopsendspeed(skype_name):
    global _Bytes_sent
    global _Start_tm
    global _RandomData
    _Bytes_sent = 0
    _Start_tm = time.time()
    connect(skype_name, _RandomData)
    reactor.callLater(0.0, _loopsendspeed, skype_name)
def cmd_sendspeed(skype_name, kilobytes):
    global _RandomData
    SetSentNotifier(_sent_func)
    _RandomData = ''
    data1024 = ''
    print 'preparing random data...'
    for i in range(1024):
        data1024 += chr(random.randint(0, 255))
    for j in range(int(kilobytes)):
        _RandomData += data1024
    print 'start sending...'
    cmd_loopsendspeed(skype_name)

#-------------------------------------------------------------------------------

def cmd_receivespeed():
    pass

def cmd_senddata(skype_name, kilobytes):
    print 'preparing random data...'
    data1024 = ''
    data = ''
    for i in range(1024):
        data1024 += chr(random.randint(0, 255))
    for j in range(int(kilobytes)):
        data += data1024
    print 'sending %s bytes ...' % str(len(data))
    connect(skype_name, data)

def cmd_status(skype_name):
    print get_user_status(skype_name)

def cmd_line_execute():
    dhnio.SetDebug(18)
    if sys.argv.count('receive'):
        init()
        cmd_receive()
    elif sys.argv.count('send'):
        init()
        cmd_send(sys.argv[2], sys.argv[3])
    elif sys.argv.count('sendshort'):
        init()
        cmd_sendshort(sys.argv[2], sys.argv[3])
    elif sys.argv.count('loopsend'):
        init()
        cmd_loopsend(sys.argv[2], sys.argv[3], int(sys.argv[4]))
    elif sys.argv.count('sendspeed'):
        init()
        cmd_sendspeed(sys.argv[2], sys.argv[3])
    elif sys.argv.count('senddata'):
        init()
        cmd_senddata(sys.argv[2], sys.argv[3])
    elif sys.argv.count('status'):
        init()
        cmd_status(sys.argv[2])
    else:
        print 'usage:'
        print 'transport_skype.py receive'
        print 'transport_skype.py receivespeed'
        print 'transport_skype.py send [skype_name] [filename]'
        print 'transport_skype.py sendshort [skype_name] [filename]'
        print 'transport_skype.py loopsend [skype_name] [filename] [timeout]'
        print 'transport_skype.py senddata [skype_name] [kilobytes]'
        print 'transport_skype.py sendspeed [skype_name] [kilobytes]'
        print 'transport_skype.py status [skype_name]'


        sys.exit()

if __name__ == '__main__':
    cmd_line_execute()
    reactor.run()









import wx
import urllib

__SENTINEL = object()

def GetParam(tag, param, default=__SENTINEL):
    """ Convenience function for accessing tag parameters"""
    if tag.HasParam(param):
        return tag.GetParam(param)
    else:
        if default == __SENTINEL:
            raise KeyError
        else:
            return default

def UnpackParam(s, default=None):
    if s is None or s == '':
        if default is not None:
            return default
        return s
    try:
        return urllib.unquote(str(s))
    except:
        return default
    
#------------------------------------------------------------------------------ 

import forms.form as form
import forms.input as input

if __name__ == '__main__':
    app = wx.App(False)
    f = wx.Frame(None)

    html = wx.html.HtmlWindow(f, style= wx.html.HW_DEFAULT_STYLE | wx.TAB_TRAVERSAL)
    html.LoadFile(r"C:\htmlt.html")

    def OnFormSubmit(evt):
        print "Submitting to %s via %s with args %s"% (evt.form.action, evt.form.method, evt.args)
    html.Bind(form.EVT_FORM_SUBMIT, OnFormSubmit)
    f.Show()
    app.MainLoop()


import wx
import wx.html


from forms import GetParam

FormSubmitEventType = wx.NewEventType()
EVT_FORM_SUBMIT = wx.PyEventBinder(FormSubmitEventType)


class FormSubmitEvent(wx.PyEvent):
    """
        Event indication a form was submitted.
        form is the form object being submitted
        args is a dict of form arguments
    """
    def __init__(self, form, args):
        wx.PyEvent.__init__(self)
        self.SetEventType(FormSubmitEventType)
        self.form = form
        self.args = args


class HTMLForm(object):
    def __init__(self, tag, container):
        self.container = container
        self.fields = []
        self.action = GetParam(tag, "ACTION", default=None)
        self.method = GetParam(tag, "METHOD", "GET")
        if self.method not in ("GET", "POST"):
            self.method = "GET"

    def hitSubmitButton(self):
        import forms.input
        for field in self.fields:
            if isinstance(field, forms.input.SubmitButton):
                field.OnClick(None)
                return

    def submit(self, btn=None):
        args = self.createArguments()
        if btn and btn.name:
            args[btn.name] = btn.GetLabel()
        evt = FormSubmitEvent(self, args)
        self.container.ProcessEvent(evt)

    def createArguments(self):
        args = {}
        for field in self.fields:
            if field.name and field.IsEnabled():
                val = field.GetValue()
                if val is None:
                    continue
                args[field.name] = val
        return args



class FormTagHandler(wx.html.HtmlWinTagHandler):
    typeregister = {}

    @classmethod
    def registerType(klass, type, controlClass):
        klass.typeregister[type] = controlClass

    def __init__(self):
        self.form = None
        wx.html.HtmlWinTagHandler.__init__(self)

    def GetSupportedTags(self):
        return "FORM,INPUT,TEXTAREA,SELECT,OPTION"

    def HandleTag(self, tag):
        try:
            handler = getattr(self, "Handle"+tag.GetName().upper())
            return handler(tag)
        except:
            import traceback
            traceback.print_exc(limit=500)

    def HandleFORM(self, tag):
        import time
        # print 'HandleFORM.start', time.time()
        self.form = HTMLForm(tag, self.GetParser().GetWindowInterface().GetHTMLWindow())
        self.cell = self.GetParser().OpenContainer()
        self.ParseInner(tag)
        self.GetParser().CloseContainer()
        self.form = None
        self.optionList = []
        # print 'HandleFORM.end', time.time()
        return True

    def HandleINPUT(self, tag):
        if tag.HasParam("type"):
            ttype = tag.GetParam("type").upper()
        else:
            ttype = "TEXT"
        klass = self.typeregister[ttype]
        obj = self.createControl(klass, tag)
        return False

    def HandleTEXTAREA(self, tag):
        klass = self.typeregister["TEXTAREA"]
        self.createControl(klass, tag)
        #Don't actually call ParseInner, but lie and said we did.
        #This should skip us ahead to the next tag, and let us
        #retrieve the text verbatem from the text area
        return True

    def HandleSELECT(self, tag):
        if tag.HasParam("MULTIPLE"):
            pass
        from forms.select import SingleSelectControl
        self.optionList = []
        #gather any/all nested options
        self.ParseInner(tag)
        parent = self.GetParser().GetWindowInterface().GetHTMLWindow()
        if 'wxMSW' in wx.PlatformInfo:
            #HAX: wxChoice has some bizarre SetSize semantics that
            #interact poorly with HtmlWidgetCell. Most notably, resizing the control
            #triggers a paint event (in the parent, I guess?) which in turn calls Layout()
            #which calls SetSize again and so on. An extra "layer" between the control
            #and the window keeps the recursion from happening.
            object = wx.Panel(parent)
            select = SingleSelectControl(object, self.form, tag, self.GetParser(), self.optionList)
            sz = wx.BoxSizer()
            sz.Add(select, 1, wx.EXPAND)
            object.SetSizer(sz)
            object.SetSize(select.GetSize())
        else:
            object = SingleSelectControl(parent, self.form, tag, self.GetParser(), self.optionList)
#        object = SingleSelectControl(parent, self.form, tag, self.GetParser(), self.optionList)
        cell = self.GetParser().GetContainer()
        cell.InsertCell(wx.html.HtmlWidgetCell(object))
        self.optionList = []
        return True

    def HandleOPTION(self, tag):
        self.optionList.append(tag)
        return True

    def createControl(self, klass, tag):
        parent = self.GetParser().GetWindowInterface().GetHTMLWindow()
        object = klass(parent, self.form, tag, self.GetParser())
        if not isinstance(object, wx.Window):
            return None
        cell = self.GetParser().GetContainer()
        cell.InsertCell(wx.html.HtmlWidgetCell(object))
        return object


wx.html.HtmlWinParser_AddTagHandler(FormTagHandler)





#!/usr/bin/python
#automat.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys

from twisted.internet import reactor
from twisted.internet.task import LoopingCall

#------------------------------------------------------------------------------ 

_Counter = 0
_Index = {}
_Objects = {}
_StateChangedCallback = None
_LogFile = None
_LogFilename = None
_LogsCount = 0
_LifeBeginsTime = 0

#------------------------------------------------------------------------------ 

def get_new_index():
    global _Counter
    _Counter += 1
    return _Counter - 1


def create_index(name):
    global _Index
    id = name
    if _Index.has_key(id):
        i = 1
        while _Index.has_key(id + '(' + str(i) + ')'):
            i += 1
        id = name + '(' + str(i) + ')'
    _Index[id] = get_new_index()
    return id, _Index[id]


def set_object(index, obj):
    global _Objects
    _Objects[index] = obj
   
    
def clear_object(index):
    global _Objects
    if _Objects is None:
        return
    if _Objects.has_key(index):
        del _Objects[index]


def objects():
    global _Objects
    return _Objects

#------------------------------------------------------------------------------ 

class Automat(object):
    timers = {}
    fast = False
    state = 'NOT_EXIST'
    def __init__(self, name, state, debug_level=18):
        self.id, self.index = create_index(name)
        self.name = name
        self.state = state
        self.debug_level = debug_level
        self._timers = {}
        self.init()
        self.startTimers()
        self.log(self.debug_level,  'NEW AUTOMAT %s CREATED with index %d' % (str(self), self.index))
        set_object(self.index, self)

    def __del__(self):
        global _Index
        global _StateChangedCallback
        if self is None:
            return
        id = self.id
        name = self.name
        debug_level = self.debug_level
        if _Index is None:
            self.log(debug_level, 'automat.__del__ Index is None')
            return
        index = _Index.get(id, None)
        if index is None:
            self.log(debug_level, 'automat.__del__ WARNING %s not found' % id)
            return
        del _Index[id]
        self.log(debug_level, 'AUTOMAT %s [%d] DESTROYED' % (id, index))
        if _StateChangedCallback is not None:
            _StateChangedCallback(index, id, name, '')

    def __repr__(self):
        return '%s[%s]' % (self.id, self.state)

    def init(self):
        pass

    def state_changed(self, oldstate, newstate):
        pass

    def A(self, event, arg):
        raise NotImplementedError

    def automat(self, event, arg=None):
        if self.fast:
            reactor.callWhenRunning(self.event, event, arg)
            # self.event(event, arg)
        else:
            reactor.callLater(0, self.event, event, arg)
            # reactor.callWhenRunning(self.event, event, arg)

    def event(self, event, arg=None):
        global _StateChangedCallback
        # self.log(self.debug_level * 4, '%s fired with event "%s"' % (self, event))# , sys.getrefcount(Automat)))
        old_state = self.state
        self.A(event, arg)
        new_state = self.state
        if old_state != new_state:
            self.stopTimers()
            self.state_changed(old_state, new_state)
            self.log(self.debug_level, '%s(%s): [%s]->[%s]' % (self.id, event, old_state, new_state))
            self.startTimers()
            if _StateChangedCallback is not None:
                _StateChangedCallback(self.index, self.id, self.name, new_state)

    def timer_event(self, name, interval):
        if self.timers.has_key(name) and self.state in self.timers[name][1]:
            self.automat(name)
        else:
            self.log(self.debug_level, '%s.timer_event ERROR timer %s not found in self.timers' % (str(self), name))

    def stopTimers(self):
        for name, timer in self._timers.items():
            if timer.running:
                timer.stop()
                # self.log(self.debug_level * 4, '%s.stopTimers timer %s stopped' % (self, name))
        self._timers.clear()

    def startTimers(self):
        for name, (interval, states) in self.timers.items():
            if len(states) > 0 and self.state not in states:
                continue
            self._timers[name] = LoopingCall(self.timer_event, name, interval)
            self._timers[name].start(interval, False)
            # self.log(self.debug_level * 4, '%s.startTimers timer %s started' % (self, name))

    def restartTimers(self):
        self.stopTimers()
        self.startTimers()

    def log(self, level, text):
        global _LogFile
        global _LogFilename
        global _LogsCount
        global _LifeBeginsTime
        if _LogFile is not None:
            import time
            if _LogsCount > 1000:
                _LogFile.close()
                _LogFile = open(_LogFilename, 'w')
                _LogsCount = 0

            s = ' ' * level + text+'\n'
            if _LifeBeginsTime != 0:
                dt = time.time() - _LifeBeginsTime
                mn = dt // 60
                sc = dt - mn * 60
                s = ('%02d:%02d.%02d' % (mn, sc, (sc-int(sc))*100)) + s
            else:
                s = time.strftime('%H:%M:%S') + s

            _LogFile.write(s)
            _LogFile.flush()
            _LogsCount += 1
        else:
            try:
                from dhnio import Dprint
                Dprint(level, text)
            except:
                try:
                    from lib.dhnio import Dprint
                    Dprint(level, text)
                except:
                    pass

#------------------------------------------------------------------------------ 

def SetStateChangedCallback(cb):
    global _StateChangedCallback
    _StateChangedCallback = cb


def OpenLogFile(filename):
    global _LogFile
    global _LogFilename
    if _LogFile:
        return
    _LogFilename = filename
    try:
        _LogFile = open(_LogFilename, 'w')
    except:
        _LogFile = None


def CloseLogFile():
    global _LogFile
    if not _LogFile:
        return
    _LogFile.flush()
    _LogFile.close()
    _LogFile = None
    _LogFilename = None


def LifeBegins(when=None):
    global _LifeBeginsTime
    if when:
        _LifeBeginsTime = when
    else:
        _LifeBeginsTime = time.time()
    


#!/usr/bin/python
#transport_cspace.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

DEBUG = True
MEMDEBUG = False

import os
import sys
import threading
import imp
import time
import socket
from struct import pack, unpack
import threading


if __name__ == '__main__':
    sys.path.insert(0, os.path.abspath('..'))
    #dirpath = os.path.dirname(os.path.abspath(__file__))
    #sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    import dhnio
    dhnio.SetDebug(20)


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in transport_cspace.py')


from zope.interface import implements
from twisted.internet import interfaces
from twisted.internet.defer import Deferred, succeed


import misc
import dhnio
import settings
import tmpfile
import contacts



NEW_USER_PASSWORD_LENGTH = 12

_EventQueue = []
_EventQueuePoolDelay = 0.2
_Status = ''
_StatusNotifyFunc = None
_ContactStatusNotifyFunc = None
_SendStatusReportFunc = None
_ReceiveStatusReportFunc = None

_State = ''
_Listener = None
_ShutdownCalled = False
_ShutdownTriggerID = None

#------------------------------------------------------------------------------

class DummyClass:
    pass

def DummyDecorator(x):
    def y(z):
        pass
    return y

def hackImport():
    sys.modules['cspace.main.incomingprompt'] = imp.new_module('cspace.main.incomingprompt')
    sys.modules['cspace.main.incomingprompt'].IncomingPromptWindow = DummyClass

    sys.modules['cspace.main.dialogs'] = imp.new_module('cspace.main.dialogs')
    sys.modules['cspace.main.dialogs'].CreateKeyDialog = DummyClass
    sys.modules['cspace.main.dialogs'].CreateKeyDoneDialog = DummyClass
    sys.modules['cspace.main.dialogs'].GoOnlineDialog = DummyClass
    sys.modules['cspace.main.dialogs'].KeyInfoDialog = DummyClass
    sys.modules['cspace.main.dialogs'].AddContactDialog = DummyClass
    sys.modules['cspace.main.dialogs'].ContactInfoDialog = DummyClass
    sys.modules['cspace.main.dialogs'].PermissionsDialog = DummyClass
    sys.modules['cspace.main.dialogs'].UpdateNotifyWindow = DummyClass

    sys.modules['cspace.main.ui'] = imp.new_module('cspace.main.ui')
    sys.modules['cspace.main.ui.Ui_MainWindow'] = imp.new_module('cspace.main.ui.Ui_MainWindow')
    sys.modules['cspace.main.ui.Ui_MainWindow'].Ui_MainWindow = DummyClass

    sys.modules['cspace.main.ui.images_rc'] = imp.new_module('cspace.main.ui.images_rc')

    sys.modules['nitro.qt4reactor'] = imp.new_module('nitro.qt4reactor')
    sys.modules['nitro.qt4reactor'].Qt4Reactor = DummyClass

    sys.modules['PyQt4'] = imp.new_module('PyQt4')
    sys.modules['PyQt4.QtCore'] = imp.new_module('PyQt4.QtCore')
    sys.modules['PyQt4'].QtCore = sys.modules['PyQt4.QtCore']
    sys.modules['PyQt4.QtGui'] = imp.new_module('PyQt4.QtGui')
    sys.modules['PyQt4'].QtGui = sys.modules['PyQt4.QtGui']

    sys.modules['PyQt4.QtCore'].pyqtSignature = DummyDecorator
    sys.modules['PyQt4.QtCore'].QSize = DummyClass
    sys.modules['PyQt4.QtCore'].SIGNAL = 0

    sys.modules['PyQt4.QtGui'].QApplication = DummyClass
    sys.modules['PyQt4.QtGui'].qApp = DummyClass
    sys.modules['PyQt4.QtGui'].QMessageBox = DummyClass
    sys.modules['PyQt4.QtGui'].QIcon = DummyClass
    sys.modules['PyQt4.QtGui'].QMainWindow = DummyClass
    sys.modules['PyQt4.QtGui'].QDialog = DummyClass
    sys.modules['PyQt4.QtGui'].QListWidgetItem = DummyClass
    sys.modules['PyQt4.QtGui'].QMenu = DummyClass
    sys.modules['PyQt4.QtGui'].QAction = DummyClass
    sys.modules['PyQt4.QtGui'].QPixmap = DummyClass

def addrToStrFixed( addr ) :
    ip = unpack( 'I', socket.inet_aton(addr[0]) )[0]
    port = addr[1]
    return pack( '!IH', ip, port )

def fix_addrToStr_bug():
    sys.modules['cspace.dht.util'].addrToStr = addrToStrFixed

#class mysocket(socket._socketobject):
#    def __init__(self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0, _sock=None):
#        print 'mysocket.__init__', family, type, proto, _sock
#        socket._socketobject.__init__(self, family, type, proto, _sock)
#        print '_sock is', self._sock
#
#    def __del__(self):
#        print 'mysocket.__del__'
#        #socket._socketobject.__del__(self)
#
#    def close(self, _closedsocket=socket._closedsocket,
#              _delegate_methods=socket._delegate_methods, setattr=setattr):
#        print 'mysocket.close', self._sock
#        socket._socketobject.close(self, _closedsocket, _delegate_methods, setattr)

#def hack_sockets():
#    sys.modules['socket'].socket = mysocket

#_ImportDone = False
#if not _ImportDone:
    #_ImportDone = True
    #hackImport()
    #import cspace_application
    #hackNCrypt()
    #fix_addrToStr_bug()
    #hack_sockets()

import cspace_application


#------------------------------------------------------------------------------

def run(init_cb=None):
    def init_callback(status):
        global _State
        dhnio.Dprint(4, 'transport_cspace.run.init_callback status=%s init_cb=%s thread is %s' % (
            status, str(init_cb), threading.currentThread().getName()))
        oldState = _State
        _State = status
        dhnio.Dprint(4, 'transport_cspace.run.init_callback State: [%s]->[%s]' % (oldState, _State))
        if init_cb is not None:
            reactor.callFromThread(init_cb, status)
    try:
        dhnio.Dprint(4, 'transport_cspace.run starting')
        #-- START CSPACE  HERE ---------------------------------------------------------
        cspace_application.run(init_callback)
        #------------------------------------------------------------------------------
        dhnio.Dprint(4, 'transport_cspace.run finished')
    except:
        dhnio.Dprint(4, 'transport_cspace.run ERROR')
        dhnio.DprintException()


def init(_resultDefer=None):
    global _State
    if _State != '':
        dhnio.Dprint(2, 'transport_cspace.init WARNING already called and current _State is '+_State)
        return succeed(_State)
    dhnio.Dprint(4, 'transport_cspace.init')

    _State = 'init'
    init_started = time.time()
    resultDefer = _resultDefer
    if resultDefer is None:
        resultDefer = Deferred()

    def waiting():
        global _State
        if time.time() - init_started > 60*2:
            dhnio.Dprint(4, 'transport_cspace.init.waiting will fire [%s] in errback' % _State)
            _State = 'timeout'
            resultDefer.errback(_State)
            return
        if _State == 'online':
            dhnio.Dprint(4, 'transport_cspace.init.waiting will fire [%s] in callback' % _State)
            resultDefer.callback(_State)
            return
        if _State == 'offline':
            dhnio.Dprint(4, 'transport_cspace.init.waiting will fire [%s] in errback' % _State)
            resultDefer.errback(_State)
            return
        reactor.callLater(0.1, waiting)

    def do_init():
        global _ShutdownTriggerID
        keyID = cspace_application.getKeyID()
        dhnio.Dprint(4, 'transport_cspace.init.do_init keyID='+keyID)

        if settings.getCSpaceKeyID() != keyID:
            dhnio.Dprint(4, 'transport_cspace.init.do_init will update user settings KeyID='+keyID)
            settings.setCSpaceKeyID(keyID)

        _ShutdownTriggerID = reactor.addSystemEventTrigger('before', 'shutdown', shutdown)

        dhnio.Dprint(4, 'transport_cspace.init.do_init will start cspace in a new thread')
        #reactor.callInThread(run, status_callback)
        reactor.callInThread(run)

    def new_key_done(x):
        dhnio.Dprint(4, 'transport_cspace.init.new_key_done: ' + str(x))
        reactor.callLater(0, do_init)

    def new_key_failed(x, resultDefer):
        dhnio.Dprint(4, 'transport_cspace.init.new_key_failed: ' + str(x))
        resultDefer.errback('not installed')

    config()

    process_events()

    waiting()

    if not cspace_application.isInstalled():
        dhnio.Dprint(4, 'transport_cspace.init want to register new key')
        d = install()
        d.addCallback(new_key_done)
        d.addErrback(new_key_failed, resultDefer)
        return resultDefer

    do_init()

    return resultDefer


def shutdown():
    global _ShutdownCalled
    if _ShutdownCalled:
        #dhnio.Dprint(2, 'transport_cspace.shutdown WARNING already called')
        return succeed('already called')
    _ShutdownCalled = True
    dhnio.Dprint(4, 'transport_cspace.shutdown')
    result = Deferred()

    def done(x):
        dhnio.Dprint(4, 'transport_cspace.shutdown.done: ' + str(x))
        global _State
        global _ShutdownCalled
        global _ShutdownTriggerID
        _State = ''
        _ShutdownCalled = False
        reactor.removeSystemEventTrigger(_ShutdownTriggerID)
        result.callback(x)

    cspace_application.stop(done)
    _State = 'shutdown'
    return result


def config():
    try:
        cspace_application.setSendNotifyFunc(onSent)
        cspace_application.setReceiveNotifyFunc(onReceive)
        cspace_application.setStatusNotifyFunc(onStatus)
        cspace_application.setContactStatusNotifyFunc(onContactStatus)

        cspace_application.setTempLocation(tmpfile.subdir('cspace-in'))

        cspace_application.hackConfigDir(getDHNConfigDir)
        cspace_application.hackPermissions()

        cspace_application._stdOutFunc = cspaceDprint
        if misc.getLocalID() == settings.CentralID():
            logLevel = 0
        else:
            logLevel = 50 - dhnio.DebugLevel * 8
            if logLevel < 0:
                logLevel = 0
        cspace_application.init_log(logLevel, settings.CSpaceLogFilename())

    except:
        dhnio.Dprint(2, 'transport_cspace.config ERROR during configuration')
        dhnio.DprintException()


def cspaceDprint(s):
    dhnio.Dprint(6, '      ' + s.strip())

#------------------------------------------------------------------------------

def push_event(e, data):
    global _EventQueue
    _EventQueue.append((e, data))

def pop_event():
    global _EventQueue
    try:
        e, d = _EventQueue.pop(0)
        
        if DEBUG:
            dhnio.Dprint(14, 'transport_cspace.process_events [%s] %s' % (e, d)) 

        if e == 'sent':
            if _SendStatusReportFunc is not None:
                _SendStatusReportFunc(d[0], d[1], d[2], d[3], d[4], d[5])

        elif e == 'received':
            tmpfile.register(d[0])
            if _ReceiveStatusReportFunc is not None:
                _ReceiveStatusReportFunc(d[0], d[1], d[2], d[3], d[4], d[5])

        elif e == 'status':
            if _StatusNotifyFunc is not None:
                _StatusNotifyFunc(d)

        elif e == 'contact-status':
            if _ContactStatusNotifyFunc is not None:
                _ContactStatusNotifyFunc(d[0], d[1])

    except:
        dhnio.DprintException()

def process_events():
    global _EventQueue
    global _EventQueuePoolDelay
    global _StatusNotifyFunc
    global _ContactStatusNotifyFunc
    global _SendStatusReportFunc
    global _ReceiveStatusReportFunc
    
    if len(_EventQueue) > 0:
        pop_event()
        _EventQueuePoolDelay = 0.01
    else:
        if _EventQueuePoolDelay < 2.0:
            _EventQueuePoolDelay *= 2.0

    # attenuation
    reactor.callLater(_EventQueuePoolDelay, process_events)

#------------------------------------------------------------------------------

def onSent(host, filename, status, proto='', error=None, message=''):
    push_event('sent', (host, filename, status, proto, error, message))


def onReceive(filename, status, proto='', host=None, error=None, message=''):
    push_event('received', (filename, status, proto, host, error, message))


def onStatus(status):
    global _Status
    if _Status != status:
        dhnio.Dprint(12, 'transport_cspace.onStatus [%s]->[%s]' % (_Status, status))
        _Status = status
        push_event('status', status)


def onContactStatus(keyid, status):
    push_event('contact-status', (keyid, status))


#------------------------------------------------------------------------------


def register(username):
    dhnio.Dprint(4, 'transport_cspace.register ' + username)
    result = Deferred()
    cspace_application.register(username)
    return succeed('registered')


def send(keyID, filename):
    #dhnio.Dprint(12, 'transport_cspace.send %s %s' % (keyID, os.path.basename(filename)))
    cspace_application.send(keyID, filename)


def receive():
    dhnio.Dprint(6, 'transport_cspace.receive')
    return succeed('ready')


def add(keyID):
    dhnio.Dprint(6, 'transport_cspace.add ' + keyID)
    resultDefer = Deferred()
    def done(c):
        if c is None:
            dhnio.Dprint(6, 'transport_cspace.add.done ERROR adding ' + keyID)
            resultDefer.errback(c)
        else:
            dhnio.Dprint(6, 'transport_cspace.add.done keyID=%s contact.name=%s' %(keyID, c.name))
            resultDefer.callback(c)
        return c
    cspace_application.add(keyID, done)
    return resultDefer


def isContact(keyID):
    return cspace_application.isContact(keyID)


def state(keyID):
    return cspace_application.state(keyID)


def status(keyID):
    return cspace_application.status(keyID)


def my_state():
    global _State
    return _State


def go_online():
    dhnio.Dprint(6, 'transport_cspace.go_online')
    return cspace_application.online()


def go_offline():
    dhnio.Dprint(6, 'transport_cspace.go_offline')
    return cspace_application.offline()


def receiving_streams(keyID):
    return cspace_application.receiving_streams(keyID)


def sending_streams(keyID):
    return cspace_application.sending_streams(keyID)


def probe(keyID):
    dhnio.Dprint(6, 'transport_cspace.probe ' + keyID)
    resultDefer = Deferred()
    def done(contact, status):
        dhnio.Dprint(6, 'transport_cspace.probe.done %s is [%s]' % (keyID, status))
        resultDefer.callback(status)
        return status
    cspace_application.probe(keyID, done)
    return resultDefer


def install(username='', password=''):
    result = Deferred()
    def done(keyID):
        dhnio.Dprint(4, 'transport_cspace.install.done keyID=' + str(keyID))
        if keyID < 0:
            result.errback(returnCode)
        else:
            result.callback(keyID)
    if username == '':
        username = misc.getIDName()
    if password == '':
        password = misc.rndstr(NEW_USER_PASSWORD_LENGTH)
    username = username.replace('-', '_')
    username = 'dhn_' + username
    dhnio.Dprint(4, 'transport_cspace.install username=' + username)
    reactor.callInThread(cspace_application.install, username, password, done)
    #cspace_application.install(username, password, done, failed)
    return result

#------------------------------------------------------------------------------

class PseudoListener:
    implements(interfaces.IListeningPort)
    def startListening(self):
        global _State
        dhnio.Dprint(4, 'transport_cspace.startListening in state ' + str(_State))
        if _State == '':
            return init()
        cspace_application.online()
        return succeed(_State)

    def stopListening(self):
        global _State
        dhnio.Dprint(4, 'transport_cspace.stopListening in state ' + str(_State))
        #return shutdown()
        cspace_application.offline()
        return succeed('offline')

    def getHost(self):
        return str(cspace_application.getKeyID())


def getListener():
    global _Listener
    if _Listener is None:
        _Listener = PseudoListener()
    return _Listener

#------------------------------------------------------------------------------

def getKeyID():
    return cspace_application.getKeyID()


def getDHNConfigDir(name):
    newDir = os.path.join(settings.BaseDir(), name.lower())
    dhnio.Dprint(6, 'transport_cspace.getDHNConfigDir [%s]=[%s]' % (name, newDir))
    return newDir

#------------------------------------------------------------------------------

def close_not_needed_streams():
    list_cspace_contacts = []
    list_streams_to_close = []
    for idurl in contacts.getContactsAndCorrespondents():
        ident = contacts.getContact(idurl)
        if ident is None:
            continue
        cspace_contact = ident.getProtoContact('cspace')
        if cspace_contact is None:
            continue
        list_cspace_contacts.append(cspace_contact[9:])
    for keyID, streams in cspace_application._SendingStreams.items():
        if keyID in list_cspace_contacts:
            continue
        for stream in streams:
            list_streams_to_close.append((stream, keyID, 'sending'))
    for keyID, streams in cspace_application._ReceivingStreams.items():
        if keyID in list_cspace_contacts:
            continue
        for stream in streams:
            list_streams_to_close.append((stream, keyID, 'receiving'))
    for stream, keyID, direction in list_streams_to_close:
        dhnio.Dprint(8, 'transport_cspace.close_not_needed_streams want to close %s stream for [%s]' % (direction, keyID))
        stream.close_stream()
    del list_cspace_contacts
    del list_streams_to_close

def SetOpenedSendingStreamsNumber(num):
    cspace_application.OPENED_SENDING_STREAMS_NUMBER = num

def SetOpenedReceivingStreamsNumber(num):
    cspace_application.OPENED_RECEIVING_STREAMS_NUMBER = num

#------------------------------------------------------------------------------

def SetStatusNotifyFunc(f):
    global _StatusNotifyFunc
    _StatusNotifyFunc = f

def SetContactStatusNotifyFunc(f):
    global _ContactStatusNotifyFunc
    _ContactStatusNotifyFunc = f

def SetSendStatusReportFunc(f):
    global _SendStatusReportFunc
    _SendStatusReportFunc = f

def SetReceiveStatusReportFunc(f):
    global _ReceiveStatusReportFunc
    _ReceiveStatusReportFunc = f

#if __name__ == '__main__':
#    def DefaultSendStatusReportFunc(a,b,c,d,e,f):
#        print 'send', a,b,c,d,e,f
#    def DefaultReceiveStatusReportFunc(a,b,c,d,e,f):
#        print 'recv', a,b,c,d,e,f
#    _SendStatusReportFunc = DefaultSendStatusReportFunc
#    _ReceiveStatusReportFunc = DefaultReceiveStatusReportFunc

#------------------------------------------------------------------------------


def test_init_done(x):
    print 'test_init_done', x
    print 'my key id is', cspace_application.getKeyID()
#    if misc.getIDName() == 'veeesel':
#        reactor.callLater(15, send, '149904', 'cspacetest.log ')
    if misc.getIDName() == 'veselin':
        reactor.callLater(15, send, '149905', sys.argv[1])
    #reactor.callLater(5, shutdown)
    return x


def usage():
    print '''
usage:
    transport_cspace.py send <keyID> <filename>
    transport_cspace.py receive
    transport_cspace.py install <username> <password>
    transport_cspace.py add <keyID>
'''


def test():
##    from cspace.dht.util import verifySignature, computeSignature
##    settings.init()
##    dhnio.SetDebug(20)
##    config()
##    p = cspace_application.profile()
##    data = open('buf/data', 'rb').read()
##    signature = open('buf/signature', 'rb').read()
##    updateLevel = int(open('buf/updateLevel', 'rb').read())
##    contact = p.getContactByName(sys.argv[2])
##    verifySignature(contact.publicKey, data, updateLevel, signature)
    cspace_application.test()


def main(argv=sys.argv):

    dhnio.LifeBegins()
    
    if argv.count('send'):
        keyID = argv[2]
        filename = argv[3]
        def DoSend(x):
            dhnio.Dprint(4, 'DoSend ' + str(x))
#            if x == 'offline':
#                reactor.stop()
#                os._exit(x)
#                return
            reactor.callLater(0, send, keyID, filename)
            reactor.callLater(10, DoSend, 0)
        def InitDone(x):
            dhnio.Dprint(4, 'InitDone [%s]' % str(x))
            if not isContact(keyID):
                add(keyID).addCallback(lambda x: probe(keyID).addCallback(DoSend))
            else:
                probe(keyID).addCallback(DoSend)
        def DoInit():
            init().addBoth(DoSend)
        settings.init()
        dhnio.SetDebug(16)
        reactor.callLater(0, DoInit)
        if MEMDEBUG:
            try:
                import memdebug
                memdebug.start(9993)
                reactor.addSystemEventTrigger('before', 'shutdown', memdebug.stop)
            except:
                pass
        reactor.run()

    elif argv.count('receive'):
        def ReceivingStarted(x):
            dhnio.Dprint(4, 'ReceivingStarted state=' + str(x))
            #reactor.callLater(3, reactor.stop)
            #print 'threads', threading.enumerate()
        settings.init()
        dhnio.SetDebug(16)
        init().addBoth(ReceivingStarted)
        if MEMDEBUG:
            try:
                import memdebug
                memdebug.start(9993)
                reactor.addSystemEventTrigger('before', 'shutdown', memdebug.stop)
            except:
                pass
        reactor.run()

    elif argv.count('install'):
        settings.init()
        dhnio.SetDebug(20)
        config()
        username = ''
        if len(argv) >= 2:
            username = argv[2]
        password = ''
        if len(argv) >= 3:
            password = argv[3]
        install(username, password)

    elif argv.count('add'):
        settings.init()
        dhnio.SetDebug(20)
        def added(x):
            dhnio.Dprint(4, 'ADDED: ' + str(x))
            reactor.stop()
            dhnio.Dprint(4, 'EXIT NOW')
            #os._exit(0)
        def doAdd(x):
            add(argv[2]).addBoth(added)
        d = init()
        d.addCallback(doAdd)
        d.addErrback(lambda x: reactor.stop())
        if MEMDEBUG:
            try:
                import memdebug
                memdebug.start(9993)
                reactor.addSystemEventTrigger('before', 'shutdown', memdebug.stop)
            except:
                pass
        reactor.run()

    elif argv.count('test'):
        test()

    else:
        usage()
        return 0

    return 0


if __name__ == '__main__':
    print 'starting', threading.enumerate()
    ret = main()
    print 'EXIT ret =', ret
    print 'finishing', threading.enumerate()
    #os._exit(ret)







#!/usr/bin/python
#delayeddelete.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


import os
import sys
import time


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in delayeddelete.py')


import dhnio
import tmpfile

_Queue = []
_QueueTask = None

#------------------------------------------------------------------------------ 

class filetodelete:
    filename = ""     # name of file we are going to soon delete
    timetodelete = 0  # time after which we should delte this file
    def __init__ (self, filename, timetodelete):
        self.filename = filename
        self.timetodelete = timetodelete


def init():
    global _Queue
    _Queue = []


def DeleteAfterTime(filename, delay):
    global _Queue
    now = time.time()
    timetodelete = now + delay
    newjob = filetodelete(filename, timetodelete)
    _Queue.append(newjob)
    CallIfNeeded()


def CheckQueue():
    global _Queue
    global _QueueTask
    
    now = time.time()
    removeditems = []
    
    for item in _Queue:
        if (item.timetodelete <= now and os.access(item.filename, os.W_OK)):
            tmpfile.throw_out(item.filename, 'delayeddelete')
            removeditems.append(item)

    if len(removeditems) > 0:
        for item in removeditems:
            _Queue.remove(item)
    del removeditems

    _QueueTask = None
    CallIfNeeded()


def CallIfNeeded():
    global _Queue
    global _QueueTask
    if _QueueTask is not None:
        return 
    if len(_Queue) > 0:
        _QueueTask = reactor.callLater(10, CheckQueue)




#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

#__all__ = ["p2p", "lib", "vertex", "epsilon", "cspace"]





#!/usr/bin/python
#transport_udp.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time
import random

from twisted.internet import reactor
from twisted.internet.task import LoopingCall 
from twisted.internet.defer import Deferred, DeferredList, succeed


import dhnio
import misc
import settings
import nameurl
import contacts
import identitycache

import stun
import shtoom.stun 
import shtoom.nat
                                                                 
import automat
import transport_udp_session 
import transport_udp_server     

_TransportUDP = None
_IsServer = False

#------------------------------------------------------------------------------ 

def init(client,):
    dhnio.Dprint(4, 'transport_udp.init ' + str(client))
    A('init', client)
    

def shutdown():
    A('stop')
    
# fast=True: put filename in the top of sending queue,
# fast=False: append to the bottom of the queue    
def send(filename, host, port, fast=False, description=''):
    # dhnio.Dprint(6, "transport_udp.send %s to %s:%s" % (os.path.basename(filename), host, port))
    A('send-file', ((host, int(port)), (filename, fast, description)))

def getListener():
    return A()

#------------------------------------------------------------------------------ 

class TransferInfo():
    def __init__(self, remote_address, filename, size):
        self.remote_address = remote_address
        self.filename = filename
        self.size = size

def current_transfers():
    return A().currentTransfers()

#------------------------------------------------------------------------------ 

def Start():
    A('start')
    
    
def Stop():
    A('stop')


def ListContactsCallback(oldlist, newlist):
    A('list-contacts', (oldlist, newlist))
    
#------------------------------------------------------------------------------ 

def SendStatusCallbackDefault(host, filename, status, proto='', error=None, message=''):
    try:
        from transport_control import sendStatusReport
        return sendStatusReport(host, filename, status, proto, error, message)
    except:
        dhnio.DprintException()
        return None

def ReceiveStatusCallbackDefault(filename, status, proto='', host=None, error=None, message=''):
    try:
        from transport_control import receiveStatusReport
        return receiveStatusReport(filename, status, proto, host, error, message)
    except:
        dhnio.DprintException()
        return None

def SendControlFuncDefault(prev_read_size, chunk_size):
    try:
        from transport_control import SendTrafficControl
        return SendTrafficControl(prev_read_size, chunk_size)
    except:
        dhnio.DprintException()
        return chunk_size
        
def ReceiveControlFuncDefault(new_data_size):
    try:
        from transport_control import ReceiveTrafficControl
        return ReceiveTrafficControl(new_data_size)
    except:
        dhnio.DprintException()
        return 0
    
def RegisterTransferFuncDefault(send_or_receive, remote_address, callback, filename, size, description):
    try:
        from transport_control import register_transfer
        return register_transfer('udp', send_or_receive, remote_address, callback, filename, size, description)
    except:
        dhnio.DprintException()
    return None
    
def UnRegisterTransferFuncDefault(transfer_id):
    try:
        from transport_control import unregister_transfer
        return unregister_transfer(transfer_id)
    except:
        dhnio.DprintException()
    return None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _TransportUDP
    if _TransportUDP is None:
        _TransportUDP = TransportUDP()
    if event is not None:
        _TransportUDP.automat(event, arg)
    return _TransportUDP

#------------------------------------------------------------------------------ 

class TransportUDP(automat.Automat):
    def __init__(self):
        self.client = None
        self.transfers = {}
        self.debug = False
        self.redirections = {}
        automat.Automat.__init__(self, 'transport_udp', 'STOPPED', 6)
        
    def A(self, event, arg):
        #---STARTED---
        if self.state is 'STARTED':
            if event == 'send-file' :
                self.doClientOutboxFile(arg)
            elif event == 'stop' :
                self.state = 'STOPPED'
                self.doShutDownAllUDPSessions(arg)
                self.doShutdownClient(arg)
            elif event == 'list-contacts' and self.isNewUDPContact(arg) :
                self.doRemoveOldUDPSession(arg)
                self.doCreateNewUDPSession(arg)
            elif event == 'child-request-remote-id' :
                self.doRequestRemoteID(arg)
            elif event == 'remote-id-received' and self.isIPPortChanged(arg) :
                self.doRestartUDPSession(arg)
            elif event == 'child-request-recreate' :
                self.doRecreateUDPSession(arg)
            elif event == 'recognize-remote-id' :
                self.doCloseDuplicatedUDPSession(arg)
        #---STOPPED---
        elif self.state is 'STOPPED':
            if event == 'init' and self.isOpenedIP(arg) :
                self.state = 'SERVER'
                self.doInitServer(arg)
            elif event == 'init' and not self.isOpenedIP(arg) :
                self.state = 'CLIENT'
                self.doInitClient(arg)
        #---SERVER---
        elif self.state is 'SERVER':
            if event == 'send-file' :
                self.doServerOutboxFile(arg)
            elif event == 'stop' :
                self.state = 'STOPPED'
        #---CLIENT---
        elif self.state is 'CLIENT':
            if event == 'start' :
                self.state = 'STARTED'
                self.doStartAllUDPSessions(arg)
            elif event == 'stop' :
                self.state = 'STOPPED'
                self.doShutdownClient(arg)

    def isOpenedIP(self, arg):
        if not arg:
            return False
        if not arg.externalAddress:
            return False
        if not arg.localAddress:
            return False            
        return arg.externalAddress[0] == arg.localAddress

    def isNewUDPContact(self, arg):
        return arg[0] != arg[1]
    
    def isIPPortChanged(self, arg):
        idurl = arg
        ident = contacts.getContact(idurl)
        if ident is None:
            return False
        address = ident.getProtoContact('udp')
        if address is None:
            return False
        try:
            ip, port = address[6:].split(':')
            address = (ip, int(port))
        except:
            dhnio.DprintException()
            return False
        address_local = self.remapAddress(address)
        found = False
        for sess in transport_udp_session.sessions():
            if  sess.remote_idurl is not None and \
                sess.remote_idurl == idurl and \
                ( sess.remote_address[0] == address[0] or sess.remote_address[0] == address_local[0] ) and \
                ( sess.remote_address[1] != address[1] and sess.remote_address[1] != address_local[1] ):
                found = True
                dhnio.Dprint(6, 'transport_udp.isIPPortChanged found related session %s with [%s]' % (sess.name, sess.remote_name))
        return found        
    
    def doInitServer(self, arg):
        self.client = arg
        transport_udp_server.init(self.client)
        # self.client.datagram_received_callback = self.datagramReceived
        self.client.datagram_received_callback = transport_udp_server.protocol().datagramReceived
        transport_udp_server.SetReceiveStatusCallback(ReceiveStatusCallbackDefault)
        transport_udp_server.SetSendStatusCallback(SendStatusCallbackDefault)
        transport_udp_server.SetReceiveControlFunc(ReceiveControlFuncDefault)
        transport_udp_server.SetSendControlFunc(SendControlFuncDefault)
        transport_udp_server.SetRegisterTransferFunc(RegisterTransferFuncDefault)
        transport_udp_server.SetUnRegisterTransferFunc(UnRegisterTransferFuncDefault)
        
    def doInitClient(self, arg):
        self.client = arg
        # self.client.datagram_received_callback = self.datagramReceived
        self.client.datagram_received_callback = transport_udp_session.data_received
        transport_udp_session.init(self)
        transport_udp_session.SetReceiveStatusCallback(ReceiveStatusCallbackDefault)
        transport_udp_session.SetSendStatusCallback(SendStatusCallbackDefault)
        transport_udp_session.SetReceiveControlFunc(ReceiveControlFuncDefault)
        transport_udp_session.SetSendControlFunc(SendControlFuncDefault)
        transport_udp_session.SetRegisterTransferFunc(RegisterTransferFuncDefault)
        transport_udp_session.SetUnRegisterTransferFunc(UnRegisterTransferFuncDefault)
        
    def doShutdownClient(self, arg):
        transport_udp_session.shutdown()
    
    def doStartAllUDPSessions(self, arg):
        all = contacts.getContactsAndCorrespondents()
        if not self.debug:
            all.append(settings.CentralID())
        for idurl in all:
            ident = contacts.getContact(idurl)
            if ident is None:
                continue
            address = ident.getProtoContact('udp')
            if address is None:
                continue
            try:
                proto, ip, port, filename = nameurl.UrlParse(address)
                address = (ip, int(port))
            except:
                dhnio.DprintException()
                continue
            address = self.remapAddress(address)
            sess = transport_udp_session.open_session(address)
            dhnio.Dprint(8, 'transport_udp.doStartAllUDPSessions init %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
            sess.automat('init', idurl)
        
    def doShutDownAllUDPSessions(self, arg):
        transport_udp_session.shutdown_all_sessions()
    
    def doRemoveOldUDPSession(self, arg):
        for idurl in arg[0]:
            if idurl == misc.getLocalID():
                continue
            if idurl not in arg[1]:
                ident = contacts.getContact(idurl)
                if ident is None:
                    continue
                address = ident.getProtoContact('udp')
                if address is None:
                    continue
                try:
                    ip, port = address[6:].split(':')
                    address = (ip, int(port))
                except:
                    dhnio.DprintException()
                    continue
                if transport_udp_session.is_session_opened(address):
                    dhnio.Dprint(8, 'transport_udp.doRemoveOldUDPSession shutdown %s with [%s]' % (str(address), nameurl.GetName(idurl)))
                    reactor.callLater(0, transport_udp_session.A, address, 'shutdown')
                address_local = self.remapAddress(address)
                if address_local != address:
                    if transport_udp_session.is_session_opened(address_local):
                        dhnio.Dprint(8, 'transport_udp.doRemoveOldUDPSession shutdown %s with local peer [%s]' % (str(address_local), nameurl.GetName(idurl)))
                        self.eraseRedirection(address_local)
                        reactor.callLater(0, transport_udp_session.A, address_local, 'shutdown')
                self.eraseRedirection(address)
                self.eraseRedirection(address_local)
                
    def doCreateNewUDPSession(self, arg):
        for idurl in arg[1]:
            if idurl == misc.getLocalID():
                continue
            if idurl not in arg[0]:
                ident = contacts.getContact(idurl)
                if ident is None:
                    continue
                address = ident.getProtoContact('udp')
                if address is None:
                    continue
                try:
                    proto, ip, port, filename = nameurl.UrlParse(address)
                    address = (ip, int(port))
                except:
                    dhnio.DprintException()
                    continue
                address = self.remapAddress(address)
                sess = transport_udp_session.open_session(address)
                dhnio.Dprint(8, 'transport_udp.doCreateNewUDPSession %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
                sess.automat('init', idurl)
    
    def doRestartUDPSession(self, arg):
        idurl = arg
        ident = contacts.getContact(idurl)
        if ident is None:
            return
        address = ident.getProtoContact('udp')
        if address is None:
            return
        try:
            ip, port = address[6:].split(':')
            address = (ip, int(port))
        except:
            dhnio.DprintException()
            return
        address_local = self.remapAddress(address)
        for sess in transport_udp_session.sessions():
            if  sess.remote_idurl is not None and sess.remote_idurl == idurl and \
                ( sess.remote_address[0] == address[0] or sess.remote_address[0] == address_local[0] ) and \
                ( sess.remote_address[1] != address[1] or sess.remote_address[1] != address_local[1] ):
                self.addRedirection(sess.remote_address, address_local)
                sess.automat('shutdown')
                dhnio.Dprint(8, 'transport_udp.doRestartUDPSession shutdown to %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
        sess = transport_udp_session.open_session(address_local)
        dhnio.Dprint(8, 'transport_udp.doRestartUDPSession init to %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
        sess.automat('init', idurl)
         
    def doRecreateUDPSession(self, arg):
        def recreate(idurl, count):
            if count > 5:
                return
            ident = contacts.getContact(idurl)
            if ident is None:
                return
            address = ident.getProtoContact('udp')
            if address is None:
                return
            try:
                ip, port = address[6:].split(':')
                address = (ip, int(port))
            except:
                dhnio.DprintException()
                return
            if transport_udp_session.is_session_opened(address):
                reactor.callLater(1, recreate, idurl, count+1)
                dhnio.Dprint(8, 'transport_udp.doRecreateUDPSession.recreate wait 1 second to finish old session with %s' % str(address))
                return
            address_local = self.remapAddress(address)
            sess = transport_udp_session.open_session(address_local)
            dhnio.Dprint(8, 'transport_udp.doRecreateUDPSession init %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
            sess.automat('init', idurl)
        reactor.callLater(random.randint(1, 10), recreate, arg, 0)
                        
    def doCloseDuplicatedUDPSession(self, arg):
        index, idurl = arg
        current_session = transport_udp_session.get_session_by_index(index)
        if current_session is None:
            dhnio.Dprint(6, 'transport_udp.doCloseDuplicatedUDPSession WARNING current session is None index=%d' % index)
            return
        for sess in transport_udp_session.sessions():
            if sess.remote_idurl is not None and sess.remote_idurl == idurl and sess.index != index:
                self.addRedirection(sess.remote_address, current_session.remote_address)
                sess.automat('shutdown')
                dhnio.Dprint(8, 'transport_udp.doCloseDuplicatedUDPSession shutdown to %s with [%s]' % (sess.name, nameurl.GetName(idurl)))
                        
    def doServerOutboxFile(self, arg):
        address = arg[0]
        filename, fast, description = arg[1]
        transport_udp_server.send(address, filename, fast, description)    
        
    def doClientOutboxFile(self, arg):
        address = arg[0]
        filename, fast, description = arg[1]
        address = self.remapAddress(address)
        if transport_udp_session.is_session_opened(address):
            transport_udp_session.outbox_file(address, filename, fast, description)
        else:
            if self.redirections.has_key(address) and transport_udp_session.is_session_opened(self.redirections[address]):
                transport_udp_session.outbox_file(self.redirections[address], filename, fast, description)
            else:
                dhnio.Dprint(6, 'transport_udp.doClientOutboxFile WARNING not found session for: ' + str(address))
                transport_udp_session.file_sent(address, filename, 'failed', 'udp', None, 'session not found')

    def doRequestRemoteID(self, arg):
        if isinstance(arg, str): 
            idurl = arg
            identitycache.immediatelyCaching(idurl).addCallbacks(
                lambda src: self.automat('remote-id-received', idurl),
                lambda x: self.automat('remote-id-failed', x))
        else:
            for idurl in arg[1]:
                if idurl == misc.getLocalID():
                    continue
                if idurl not in arg[0]:
                    identitycache.immediatelyCaching(idurl).addCallbacks(
                        lambda src: self.automat('remote-id-received', idurl),
                        lambda x: self.automat('remote-id-failed', x))

    def stopListening(self):
        dhnio.Dprint(4, 'transport_udp.stopListening')
        self.automat('stop')
        res = stun.stopUDPListener()
        if res is None:
            res = succeed(1)
        return res

    # use local IP instead of external if it comes from central server 
    def remapAddress(self, address):
        return identitycache.RemapContactAddress(address)
    
    def addRedirection(self, old_address, new_address):
        self.redirections[old_address] = new_address
    
    def eraseRedirection(self, address_to_close):
        to_remove = []
        for old_address, new_address in self.redirections.items():
            if new_address == address_to_close:
                to_remove.append(old_address)
        for address_to_close in to_remove:
            del self.redirections[address_to_close]
        del to_remove
                    
    
#------------------------------------------------------------------------------ 

def main():
    sys.path.append('..')

    def _go_stun(port):
        print '+++++ LISTEN UDP ON PORT', port, 'AND RUN STUN DISCOVERY'
        stun.stunExternalIP(close_listener=False, internal_port=port, verbose=False).addBoth(_stuned)

    def _stuned(ip):
        if stun.getUDPClient() is None:
            print 'UDP CLIENT IS NONE - EXIT'
            reactor.stop()
            return

        print '+++++ EXTERNAL UDP ADDRESS IS', stun.getUDPClient().externalAddress
        
        if sys.argv[1] == 'listen':
            print '+++++ START LISTENING'
            return
        
        if sys.argv[1] == 'connect':
            print '+++++ CONNECTING TO REMOTE MACHINE'
            _try2connect()
            return

        lid = misc.getLocalIdentity()
        udp_contact = 'udp://'+stun.getUDPClient().externalAddress[0]+':'+str(stun.getUDPClient().externalAddress[1])
        lid.setProtoContact('udp', udp_contact)
        lid.sign()
        misc.setLocalIdentity(lid)
        misc.saveLocalIdentity()
        
        print '+++++ UPDATE IDENTITY', str(lid.contacts)
        _send_servers().addBoth(_id_sent)

    def _send_servers():
        import tmpfile, misc, nameurl, settings, transport_tcp
        sendfile, sendfilename = tmpfile.make("propagate")
        os.close(sendfile)
        LocalIdentity = misc.getLocalIdentity()
        dhnio.WriteFile(sendfilename, LocalIdentity.serialize())
        dlist = []
        for idurl in LocalIdentity.sources:
            # sources for out identity are servers we need to send to
            protocol, host, port, filename = nameurl.UrlParse(idurl)
            port = settings.IdentityServerPort()
            d = Deferred()
            transport_tcp.send(sendfilename, host, port, do_status_report=False, result_defer=d, description='Identity')
            dlist.append(d) 
        dl = DeferredList(dlist, consumeErrors=True)
        print '+++++ IDENTITY SENT TO', str(LocalIdentity.sources)
        return dl

    def _try2connect():
        remote_addr = dhnio.ReadTextFile(sys.argv[3]).split(' ')
        remote_addr = (remote_addr[0], int(remote_addr[1]))
        t = int(str(int(time.time()))[-1]) + 1
        data = '0' * t
        stun.getUDPClient().transport.write(data, remote_addr)
        print 'sent %d bytes to %s' % (len(data), str(remote_addr))
        reactor.callLater(1, _try2connect)

    def _id_sent(x):
        print '+++++ ID UPDATED ON THE SERVER', x
        if sys.argv[1] == 'send':
            _start_sending()
        elif sys.argv[1] == 'receive':
            _start_receiving()

    def _start_receiving():
        idurl = sys.argv[2]
        if not idurl.startswith('http://'):
            idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        print '+++++ START RECEIVING FROM', idurl
        _request_remote_id(idurl).addBoth(_receive_from_remote_peer, idurl)
        
    def _receive_from_remote_peer(x, idurl):
        init(stun.getUDPClient())
        A().debug = True
        contacts.addCorrespondent(idurl)
        reactor.callLater(1, Start)

    def _start_sending():
        idurl = sys.argv[2]
        if not idurl.startswith('http://'):
            idurl = 'http://'+settings.IdentityServerName()+'/'+idurl+'.xml'
        print '+++++ START SENDING TO', idurl
#        if len(sys.argv) == 6:
#            send(sys.argv[5], sys.argv[3], int(sys.argv[4]))
#        elif len(sys.argv) == 4:
        _request_remote_id(idurl).addBoth(_send_to_remote_peer, idurl, sys.argv[3], None if len(sys.argv)<5 else int(sys.argv[4]))

    def _request_remote_id(idurl):
        print '+++++ REQUEST ID FROM SERVER', idurl
        return identitycache.immediatelyCaching(idurl)
    
    def _send_to_remote_peer(x, idurl, filename, loop_delay):
        print '+++++ PREPARE SENDING TO', idurl
        init(stun.getUDPClient())
        A().debug = True
        contacts.addCorrespondent(idurl)
        reactor.callLater(1, Start)
        if loop_delay:
            LoopingCall(_send_file, idurl, filename,).start(loop_delay, False)
        else:
            reactor.callLater(2, _send_file, idurl, filename,)
    
    def _send_file(idurl, filename):
        ident = identitycache.FromCache(idurl)
        if ident is None:
            print '+++++ REMOTE IDENTITY IS NONE'
            reactor.stop()
        x, udphost, udpport, x = ident.getProtoParts('udp')
        print '+++++ SENDING TO', udphost, udpport
        send(filename, udphost, udpport)
    
    dhnio.SetDebug(20)
    dhnio.LifeBegins()
    settings.init()
    misc.loadLocalIdentity()
    # contacts.init()
    # contacts.addCorrespondent(idurl)
    identitycache.init()
    port = int(settings.getUDPPort())
    if sys.argv[1] in ['listen', 'connect']:
        port = int(sys.argv[2])
    _go_stun(port)

#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    if len(sys.argv) not in [2, 3, 4, 5, 6] or sys.argv[1] not in ['send', 'receive', 'listen', 'connect']:
        print 'transport_udp.py receive <from username>'
        print 'transport_udp.py receive <from idurl>'
        print 'transport_udp.py send <to username> <filename>'
        print 'transport_udp.py send <to idurl> <filename>'
        print 'transport_udp.py send <to username> <filename> <loop delay>'
        print 'transport_udp.py send <to idurl> <filename> <loop delay>'
        print 'transport_udp.py listen <listening port>'
        print 'transport_udp.py connect <listening port> <filename with remote address>'
        sys.exit(0)
        
    main()
    reactor.run()



